/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.perseus.dependency.api;

/**
 * Manage a dependency graph between object.
 *
 * @author S.Chassande-Barrioz
 */
public interface DependencyGraph {
	/**
	 * Add a vertex between to task. If the added vertex creates a cycle, it is
	 * not added, and the 'false' value is returned.
	 *
	 * @param src is the source of the vertex
	 * @param dst is the target of the vertex
	 * @return 'true' is the vertex has been added. 'false' if the vertex would
	 * create a cycle if it has been added.
	 */
	boolean addVertex(Object src, Object dst);

	/**
	 * Removes the vertex/dependency between two tasks.
	 * 
	 * @param src is the source of the vertex
	 * @param dst is the target of the vertex
	 */
	void removeVertex(Object src, Object dst);
}
