/**
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.perseus.dependency.lib;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.perseus.dependency.api.DependencyGraph;
import org.objectweb.util.monolog.api.Logger;

import java.util.*;


/**
 *
 * @author S.Chassande-Barrioz
 */
public final class BasicDependencyGraph
        implements DependencyGraph, BindingController{

    // associates to each node the set of its successors in the graph
    public Map successors;

    // list of nodes to be visited
    private List toBeVisited;

    // list of nodes already visited
    private List visited;

    protected Logger logger = null;

    public BasicDependencyGraph () {
        successors = new HashMap();
        toBeVisited = new ArrayList();
        visited = new ArrayList();
    }

    //IMPLEMENTATION OF THE UserBindingControler INTERFACE //
    //------------------------------------------------------//

    public String[] listFc() {
        return new String[] {};
    }

    public Object lookupFc(String s) {
        return null;
    }

    public void bindFc(String s, Object o) {
        if ("logger".equals(s)) {
            logger = (Logger) o;
        }
    }

    public void unbindFc(String s) {
    }


    //IMPLEMENTATION OF THE DependencyGraph INTERFACE //
    //------------------------------------------------//

    /**
     * Adds a vertex in the graph, or returns false is this would create a
     * cycle.
     */
    public synchronized boolean addVertex (Object src, Object dst) {
        Set s = (Set)successors.get(src);
        if (s == null) {
            s = new HashSet();
            successors.put(src, s);
        }
        if (src.equals(dst) || s.contains(dst)) {
            return true;
        }

        toBeVisited.clear();
        visited.clear();

        // finds the nodes accessible from dst
        toBeVisited.add(dst);
        while (!toBeVisited.isEmpty()) {
            Object node = toBeVisited.remove(toBeVisited.size() - 1);
            if (node.equals(src)) {
                // src is accessible from dst
                // adding a vertex [src -> dst] would create a cycle
                return false;
            }
            visited.add(node);

            // adds the successors of 'node' in toBeVisited
            Set t = (Set)successors.get(node);
            if (t != null) {
                Iterator i = t.iterator();
                while (i.hasNext()) {
                    Object successor = i.next();
                    if (!visited.contains(successor)) {
                        toBeVisited.add(successor);
                    }
                }
            }
        }

        s.add(dst);
        return true;
    }

    /**
     * Removes a vertex in the graph.
     */
    public synchronized void removeVertex (Object src, Object dst) {
        Set s = (Set)successors.get(src);
        if (s != null) {
            s.remove(dst);
            if (s.isEmpty()) {
                successors.remove(src);
            }
        }
    }

    void print(Object src, Object dst) {
        StringBuffer sb = new StringBuffer();
        sb.append("src=");
        sb.append(src);
        sb.append("\ndst=");
        sb.append(dst);
        sb.append("\n");
        sb.append("dependency Graph: ");
        if (successors.size() > 0) {
            List waiters = new ArrayList(successors.keySet());
            Set s = new HashSet(waiters.size() * 2);
            s.addAll(successors.keySet());
            s.addAll(successors.values());
            for (int i = 0; i < waiters.size(); i++) {
                sb.append("\n");
                sb.append(waiters.get(i));
                sb.append(" = > ");
                sb.append(((Set)successors.get(waiters.get(i))).size());
                sb.append(": ");
                sb.append(successors.get(waiters.get(i)));
            }
        }
        System.out.println(sb.toString());
    }

}
